Package: lintr
Title: A 'Linter' for R Code
Version: 3.4.0
Authors@R: c(
    person("Jim", "Hester", , role = "aut"),
    person("Florent", "Angly", role = "aut",
           comment = c(GitHub = "fangly")),
    person("Russ", "Hyde", role = "aut"),
    person("Michael", "Chirico", email = "michaelchirico4@gmail.com", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0003-0787-087X")),
    person("Kun", "Ren", role = "aut"),
    person("Alexander", "Rosenstock", role = "aut",
           comment = c(GitHub = "AshesITR")),
    person("Indrajeet", "Patil", email = "patilindrajeet.science@gmail.com", role = "aut",
           comment = c(ORCID = "0000-0003-1995-6531")),
    person("Hugo", "Gruson", role = "aut",
           comment = c(ORCID = "0000-0002-4094-1476")),
    person("Emmanuel", "Ferdman", role = "aut")
  )
Description: Checks adherence to a given style, syntax errors and possible
    semantic issues.  Supports on the fly checking of R code edited with
    'RStudio IDE', 'Emacs', 'Vim', 'Sublime Text', 'Atom' and 'Visual
    Studio Code'.
License: MIT + file LICENSE
URL: https://lintr.r-lib.org, https://github.com/r-lib/lintr
BugReports: https://github.com/r-lib/lintr/issues
Depends: R (>= 4.1.0)
Imports: cli (>= 3.4.0), codetools, digest, glue, knitr, rex, stats,
        utils, xfun, xml2 (>= 1.3.4), xmlparsedata (>= 1.0.5)
Suggests: bookdown, cyclocomp, jsonlite, patrick (>= 0.2.0), rlang,
        rmarkdown, rstudioapi (>= 0.2), testthat (>= 3.3.0), tibble,
        tufte, withr (>= 2.5.0)
Enhances: data.table
VignetteBuilder: knitr
Config/Needs/website: tidyverse/tidytemplate
Config/Needs/development: pkgload, testthat, patrick
Config/testthat/edition: 3
Encoding: UTF-8
Collate: 'make_linter_from_xpath.R' 'xp_utils.R' 'utils.R' 'AAA.R'
        'T_and_F_symbol_linter.R' 'absolute_path_linter.R' 'actions.R'
        'addins.R' 'all_equal_linter.R' 'any_duplicated_linter.R'
        'any_is_na_linter.R' 'assignment_linter.R' 'backport_linter.R'
        'boolean_arithmetic_linter.R' 'brace_linter.R' 'cache.R'
        'class_equals_linter.R' 'coalesce_linter.R' 'commas_linter.R'
        'commented_code_linter.R' 'comparison_negation_linter.R'
        'condition_call_linter.R' 'condition_message_linter.R'
        'conjunct_test_linter.R' 'consecutive_assertion_linter.R'
        'consecutive_mutate_linter.R' 'cyclocomp_linter.R'
        'declared_functions.R' 'deprecated.R' 'download_file_linter.R'
        'duplicate_argument_linter.R' 'empty_assignment_linter.R'
        'equals_na_linter.R' 'exclude.R' 'expect_comparison_linter.R'
        'expect_identical_linter.R' 'expect_length_linter.R'
        'expect_lint.R' 'expect_named_linter.R' 'expect_not_linter.R'
        'expect_null_linter.R' 'expect_s3_class_linter.R'
        'expect_s4_class_linter.R' 'expect_true_false_linter.R'
        'expect_type_linter.R' 'extract.R' 'fixed_regex_linter.R'
        'for_loop_index_linter.R' 'function_argument_linter.R'
        'function_left_parentheses_linter.R' 'function_return_linter.R'
        'get_source_expressions.R' 'ids_with_token.R'
        'if_not_else_linter.R' 'if_switch_linter.R'
        'ifelse_censor_linter.R' 'implicit_assignment_linter.R'
        'implicit_integer_linter.R' 'indentation_linter.R'
        'infix_spaces_linter.R' 'inner_combine_linter.R'
        'is_lint_level.R' 'is_numeric_linter.R'
        'keyword_quote_linter.R' 'length_levels_linter.R'
        'length_test_linter.R' 'lengths_linter.R'
        'library_call_linter.R' 'line_length_linter.R' 'lint.R'
        'linter_tag_docs.R' 'linter_tags.R' 'lintr-package.R'
        'list2df_linter.R' 'list_comparison_linter.R'
        'literal_coercion_linter.R' 'make_linter_from_regex.R'
        'matrix_apply_linter.R' 'methods.R' 'missing_argument_linter.R'
        'missing_package_linter.R' 'namespace.R' 'namespace_linter.R'
        'nested_ifelse_linter.R' 'nested_pipe_linter.R'
        'nonportable_path_linter.R' 'shared_constants.R'
        'nrow_subset_linter.R' 'numeric_leading_zero_linter.R'
        'nzchar_linter.R' 'object_length_linter.R'
        'object_name_linter.R' 'object_overwrite_linter.R'
        'object_usage_linter.R' 'one_call_pipe_linter.R'
        'outer_negation_linter.R' 'package_hooks_linter.R'
        'paren_body_linter.R' 'paste_linter.R' 'path_utils.R'
        'pipe_call_linter.R' 'pipe_consistency_linter.R'
        'pipe_continuation_linter.R' 'pipe_return_linter.R'
        'print_linter.R' 'quotes_linter.R' 'redundant_equals_linter.R'
        'redundant_ifelse_linter.R' 'regex_subset_linter.R'
        'rep_len_linter.R' 'repeat_linter.R' 'return_linter.R'
        'routine_registration_linter.R' 'sample_int_linter.R'
        'scalar_in_linter.R' 'semicolon_linter.R' 'seq_linter.R'
        'settings.R' 'settings_utils.R' 'sort_linter.R'
        'source_utils.R' 'spaces_inside_linter.R'
        'spaces_left_parentheses_linter.R' 'sprintf_linter.R'
        'stopifnot_all_linter.R' 'string_boundary_linter.R'
        'strings_as_factors_linter.R' 'system_file_linter.R'
        'terminal_close_linter.R' 'todo_comment_linter.R'
        'trailing_blank_lines_linter.R' 'trailing_whitespace_linter.R'
        'tree_utils.R' 'undesirable_function_linter.R'
        'undesirable_operator_linter.R'
        'unnecessary_concatenation_linter.R'
        'unnecessary_lambda_linter.R' 'unnecessary_nesting_linter.R'
        'unnecessary_placeholder_linter.R' 'unreachable_code_linter.R'
        'unused_import_linter.R' 'use_lintr.R' 'vector_logic_linter.R'
        'which_grepl_linter.R' 'whitespace_linter.R' 'with.R'
        'with_id.R' 'xml_nodes_to_lints.R' 'xml_utils.R'
        'yoda_test_linter.R' 'zzz.R'
Language: en-US
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-07-15 23:20:21 UTC; chiricom
Author: Jim Hester [aut],
  Florent Angly [aut] (GitHub: fangly),
  Russ Hyde [aut],
  Michael Chirico [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-0787-087X>),
  Kun Ren [aut],
  Alexander Rosenstock [aut] (GitHub: AshesITR),
  Indrajeet Patil [aut] (ORCID: <https://orcid.org/0000-0003-1995-6531>),
  Hugo Gruson [aut] (ORCID: <https://orcid.org/0000-0002-4094-1476>),
  Emmanuel Ferdman [aut]
Maintainer: Michael Chirico <michaelchirico4@gmail.com>
Repository: CRAN
Date/Publication: 2026-07-16 00:10:02 UTC
