Package: assignmentLinter
Type: Package
Title: What the package does (short line)
Version: 1.0
Date: 2019-12-17
Author: Who wrote it
Maintainer: Who to complain to <yourfault@somewhere.net>
Description: More about what it does (maybe more than one line)
License: What license is it under?
Suggests: 
    testthat (>= 3.0.0)
Config/testthat/edition: 3
