# Generated by roxygen2: do not edit by hand

S3method("names<-",my_custom_class)
S3method(drink_me,data.frame)
S3method(drink_me,default)
S3method(drink_me,list)
S3method(eat_me,liiiiiiiiiiiiiiiiiiiiiiiiiiist)
S3method(head,my_s3_object)
export(drink_me)
export(eat_me)
importFrom(utils,head)
